package cn.com.qmth.stmms.admin.report;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.admin.exam.BaseExamController;
import cn.com.qmth.stmms.biz.report.model.ReportSubject;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectService;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;

/**
 * 课程总量统计
 *
 * @author xuwenjin
 * @date 2019-04-12 17:05
 */
@Controller
@RequestMapping("/admin/exam/reportSubject")
public class ReportSubjectController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(ReportSubjectController.class);

    @Autowired
    private ReportSubjectService reportSubjectService;

    @RequestMapping
    public String list(Model model, HttpServletRequest request, ReportSubjectQuery query) {
        WebUser webuser = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        query = reportSubjectService.findByQuery(query);
        if (query.getCurrentCount() > 0) {
            model.addAttribute("list", query.getResult());
        }
        model.addAttribute("subjectList", getExamSubject(examId, webuser));
        model.addAttribute("query", query);
        return "modules/report/reportSubject";
    }

    @RequestMapping("/export")
    public String export(ReportSubjectQuery query, HttpServletRequest request, HttpServletResponse response,
            RedirectAttributes redirectAttributes) {
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        query = reportSubjectService.findByQuery(query);
        List<ReportSubject> list = query.getResult();
        for (ReportSubject r : list) {
            r.setAvgScore(new BigDecimal(r.getAvgScore()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setPassRate(new BigDecimal(r.getPassRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
        }
        String fileName = "课程总量统计.xlsx";
        try {
            new ExportExcel("课程总量统计", ReportSubject.class).setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导出成绩失败！" + e.getMessage());
            return "redirect:/admin/exam/reportSubject";
        }
    }

}
