package cn.com.qmth.stmms.admin.report;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.admin.exam.BaseExamController;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectQuestion;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectQuestionService;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;

/**
 * 客、主观题 成绩分析
 *
 * @author xuwenjin
 * @date 2019-04-18
 */
@Controller
@RequestMapping("/admin/exam/reportSubjectQuestion")
public class ReportSubjectQuestionController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(ReportSubjectQuestionController.class);

    @Autowired
    private ReportSubjectQuestionService reportSubjectQuestionService;
    
    @Autowired
    private ExamQuestionService questionService;

    @RequestMapping
    public String list(Model model, HttpServletRequest request, ReportSubjectQuery query) {
        WebUser webuser = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        if (StringUtils.isNotBlank(query.getSubjectCode())&&query.getObjective()!=null) {
            List<ReportSubjectQuestion> questions = reportSubjectQuestionService.findByQuery(query);
            model.addAttribute("list", questions);
            model.addAttribute("typeList", questionService.getPaperType(examId, query.getSubjectCode()));
        }
        model.addAttribute("subjectList", getExamSubject(examId, webuser));
        model.addAttribute("query", query);
        return "modules/report/reportSubjectQuestion";
    }

    @RequestMapping("/export")
    public String export(ReportSubjectQuery query, HttpServletRequest request, HttpServletResponse response,
            RedirectAttributes redirectAttributes) {
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        List<ReportSubjectQuestion> list = reportSubjectQuestionService.findByQuery(query);
        for (ReportSubjectQuestion r : list) {
            r.setAvgScore(new BigDecimal(r.getAvgScore()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setStdev(new BigDecimal(r.getStdev()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setScoreRate(new BigDecimal(r.getScoreRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setFullScoreRate(new BigDecimal(r.getFullScoreRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
        }
        String fileName = query.getObjective()?"客观题统计分析.xlsx":"主观题统计分析.xlsx";
        try {
            new ExportExcel(query.getObjective()?"客观题统计分析":"主观题统计分析", ReportSubjectQuestion.class).setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导出成绩失败！" + e.getMessage());
            return "redirect:/admin/exam/reportSubjectQuestion";
        }
    }
}
