package cn.com.qmth.stmms.api.controller;

import java.text.DecimalFormat;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.google.common.base.Strings;

import cn.com.qmth.stmms.api.utils.AESUtil;
import cn.com.qmth.stmms.api.utils.MenualAbsentDTO;
import cn.com.qmth.stmms.biz.api.auth.annotation.AuthValidate;
import cn.com.qmth.stmms.biz.api.auth.exception.ApiException;
import cn.com.qmth.stmms.biz.campus.model.Campus;
import cn.com.qmth.stmms.biz.campus.service.CampusService;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.utils.ScoreItem;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

@Controller("examStudentApiController")
@RequestMapping("/api")
public class ExamStudentController extends BaseApiController {

    protected static Logger logger = LoggerFactory.getLogger(ExamStudentController.class);

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamService examService;

    @Autowired
    private CampusService campusService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private MarkerService markerService;

    @AuthValidate("adminUser")
    @RequestMapping(value = "/student/manualAbsent/{examId}", method = RequestMethod.POST)
    @ResponseBody
    public Object updateManualAbsent(HttpServletRequest request, @PathVariable Integer examId,
            @RequestBody MenualAbsentDTO[] datas) {
        User user = RequestUtils.getApiUser(request);
        Exam exam = examService.findById(examId);
        if (exam != null && exam.getSchoolId().equals(user.getSchoolId())) {
            for (MenualAbsentDTO dto : datas) {
                studentService.updateManualAbsent(examId, dto.getExamNumber(), dto.isAbsent());
            }
            return true;
        } else {
            throw ApiException.EXAM_NOT_ACCESSIBLED;
        }
    }

    @AuthValidate("adminUser")
    @RequestMapping(value = "/student/manualAbsent/clear", method = RequestMethod.POST)
    @ResponseBody
    public Object clearManualAbsent(HttpServletRequest request, @RequestParam Integer examId) {
        User user = RequestUtils.getApiUser(request);
        Exam exam = examService.findById(examId);
        if (exam != null && exam.getSchoolId().equals(user.getSchoolId())) {
            studentService.clearManualAbsent(examId);
            return true;
        } else {
            throw ApiException.EXAM_NOT_ACCESSIBLED;
        }
    }

    @AuthValidate({ "adminUser", "scanner" })
    @RequestMapping(value = "/exam/students/{examId}", method = RequestMethod.GET)
    @ResponseBody
    public JSONArray getExamStudents(HttpServletRequest request, @PathVariable Integer examId) {
        JSONArray array = new JSONArray();
        List<ExamStudent> esList = new LinkedList<ExamStudent>();
        Exam exam = examService.findById(examId);
        if (exam != null) {
            esList = studentService.findByExamId(examId);
        }
        for (ExamStudent student : esList) {
            JSONObject obj = new JSONObject();
            obj.accumulate("examNumber", student.getExamNumber());
            obj.accumulate("campusName", student.getCampusName());
            obj.accumulate("subjectCode", student.getSubjectCode());
            obj.accumulate("subjectName", student.getSubjectName());
            obj.accumulate("name", student.getName());
            obj.accumulate("studentId", String.valueOf(student.getId()));
            obj.accumulate("barcode", student.getExamNumber());

            Campus campus = campusService.findBySchoolAndName(exam.getSchoolId(), student.getCampusName());
            obj.accumulate("campusCode", campus != null ? campus.getId().toString() : "");
            array.add(obj);
        }
        return array;
    }

    @AuthValidate({ "adminUser", "scanner" })
    @RequestMapping(value = "/students/{examId}", method = RequestMethod.GET)
    @ResponseBody
    public JSONArray getStudent(HttpServletRequest request, @PathVariable Integer examId,
            @RequestParam(required = false) Boolean upload, @RequestParam(required = false) Boolean absent,
            @RequestParam(required = false) Integer pageNumber, @RequestParam(required = false) Integer pageSize) {
        JSONArray array = new JSONArray();
        Exam exam = examService.findById(examId);
        if (exam == null) {
            return array;
        }

        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setExamId(examId);
        query.setUpload(upload);
        query.setAbsent(absent);
        if (pageNumber != null && pageSize != null) {
            query.setPageNumber(pageNumber);
            query.setPageSize(pageSize);
        } else {
            query.setPageNumber(1);
            query.setPageSize(Integer.MAX_VALUE);
        }
        studentService.findByQuery(query);
        for (ExamStudent student : query.getResult()) {
            JSONObject obj = new JSONObject();
            obj.accumulate("id", student.getId());
            obj.accumulate("examNumber", student.getExamNumber());
            obj.accumulate("campusName", student.getCampusName());
            obj.accumulate("subjectCode", student.getSubjectCode());
            obj.accumulate("subjectName", student.getSubjectName());
            obj.accumulate("name", student.getName());
            obj.accumulate("studentCode", student.getStudentCode());

            Campus campus = campusService.findBySchoolAndName(exam.getSchoolId(), student.getCampusName());
            obj.accumulate("campusCode", campus != null ? campus.getId().toString() : "");
            array.add(obj);
        }
        return array;
    }

    @AuthValidate({ "adminUser", "scanner" })
    @RequestMapping(value = "/student/check", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject checkStudent(HttpServletRequest request, @RequestBody ExamStudent examStudent) {
        JSONObject obj = new JSONObject();
        Exam exam = examService.findById(examStudent.getExamId());
        ExamStudent student = studentService.findByExamIdAndExamNumber(examStudent.getExamId(),
                examStudent.getExamNumber());
        if (student != null) {
            obj.accumulate("examId", examStudent.getExamId());
            obj.accumulate("campusName", student.getCampusName());
            obj.accumulate("examNumber", student.getExamNumber());
            obj.accumulate("name", student.getName());
            obj.accumulate("studentId", String.valueOf(student.getId()));
            obj.accumulate("subjectCode", student.getSubjectCode());
            obj.accumulate("subjectName", student.getSubjectName());

            Campus campus = campusService.findBySchoolAndName(exam.getSchoolId(), student.getCampusName());
            obj.accumulate("campusCode", campus != null ? campus.getId().toString() : "");
        } else {
            obj.accumulate("examId", examStudent.getExamId());
            obj.accumulate("campusCode", "");
            obj.accumulate("campusName", "");
            obj.accumulate("examNumber", examStudent.getExamNumber());
            obj.accumulate("name", "");
            obj.accumulate("studentId", "");
            obj.accumulate("subjectCode", "");
            obj.accumulate("subjectName", "");
        }
        return obj;
    }

    @AuthValidate("adminUser")
    @RequestMapping("/students/count/{examId}")
    @ResponseBody
    public long getStudentCount(HttpServletRequest request, @PathVariable Integer examId,
            @RequestParam(required = false) Boolean upload, @RequestParam(required = false) Boolean absent) {
        User user = RequestUtils.getApiUser(request);
        Exam exam = examService.findById(examId);
        if (exam != null && exam.getSchoolId().equals(user.getSchoolId())) {
            ExamStudentSearchQuery query = new ExamStudentSearchQuery();
            query.setExamId(examId);
            query.setUpload(upload);
            query.setAbsent(absent);
            return studentService.countByQuery(query);
        }
        return 0;
    }

    @AuthValidate("adminUser")
    @RequestMapping("/exam/students")
    @ResponseBody
    public JSONArray getStudent(HttpServletRequest request, ExamStudentSearchQuery query,
            @RequestParam(required = false) Boolean withScoreDetail,
            @RequestParam(required = false) Boolean withMarkTrack) {
        User user = RequestUtils.getApiUser(request);
        JSONArray array = new JSONArray();
        if (query.getExamId() == null) {
            return array;
        }
        Exam exam = examService.findById(query.getExamId());
        if (exam != null && exam.getSchoolId().equals(user.getSchoolId())) {
            DecimalFormat format = new DecimalFormat("####.##");
            studentService.findByQuery(query);
            for (ExamStudent student : query.getResult()) {
                JSONObject obj = new JSONObject();
                obj.accumulate("id", student.getId());
                obj.accumulate("schoolId", student.getSchoolId());
                obj.accumulate("examNumber", student.getExamNumber());
                obj.accumulate("campusName", student.getCampusName());
                obj.accumulate("subjectCode", student.getSubjectCode());
                obj.accumulate("subjectName", student.getSubjectName());
                obj.accumulate("name", student.getName());
                obj.accumulate("studentCode", student.getStudentCode());
                obj.accumulate("packageCode", student.getPackageCode());
                obj.accumulate("batchCode", student.getBatchCode() == null ? "" : student.getBatchCode());
                obj.accumulate("sheetCount", student.getSheetCount() != null ? student.getSheetCount() : 0);
                obj.accumulate("sliceCount", student.getSliceCount() != null ? student.getSliceCount() : 0);
                obj.accumulate("answers", StringUtils.trimToEmpty(student.getAnswers()));
                obj.accumulate("upload", student.isUpload());
                obj.accumulate("absent", student.isAbsent());
                obj.accumulate("manualAbsent", student.isManualAbsent());
                obj.accumulate("breach", student.isBreach());
                obj.accumulate("objectiveScore",
                        student.getObjectiveScore() == null ? "" : format.format(student.getObjectiveScore()));
                obj.accumulate("subjectiveScore",
                        student.getSubjectiveScore() == null ? "" : format.format(student.getSubjectiveScore()));
                obj.accumulate("examSite", StringUtils.trimToEmpty(student.getExamSite()));
                obj.accumulate("examRoom", StringUtils.trimToEmpty(student.getExamRoom()));
                obj.accumulate("remark", StringUtils.trimToEmpty(student.getRemark()));
                Campus campus = campusService.findBySchoolAndName(student.getSchoolId(), student.getCampusName());
                obj.accumulate("campusCode", campus != null ? campus.getId().toString() : "");

                try {
                    if (withScoreDetail != null && withScoreDetail.booleanValue()) {
                        // 构造客观题得分明细
                        JSONArray objective = new JSONArray();
                        List<ScoreItem> scoreList = student.getScoreList(true);
                        List<ExamQuestion> questionList = questionService.findByExamAndSubjectAndObjectiveAndPaperType(
                                student.getExamId(), student.getSubjectCode(), true, student.getPaperType());
                        int i = 0;
                        for (ScoreItem item : scoreList) {
                            i++;
                            if (questionList.size() < i) {
                                break;
                            }
                            ExamQuestion question = questionList.get(i - 1);
                            if (question.getTotalScore() == null || question.getTotalScore() == 0) {
                                continue;
                            }
                            JSONObject detail = new JSONObject();
                            detail.accumulate("mainNumber", question.getMainNumber());
                            detail.accumulate("subNumber", question.getSubNumber());
                            detail.accumulate("score", item.getScore());
                            detail.accumulate("answer", item.getAnswer());
                            objective.add(detail);
                        }
                        obj.accumulate("objectiveScoreDetail", objective);

                        // 构造主观题得分明细
                        JSONArray subjective = new JSONArray();
                        scoreList = student.getScoreList(false);
                        questionList = questionService.findByExamAndSubjectAndObjective(student.getExamId(),
                                student.getSubjectCode(), false);
                        i = 0;
                        for (ScoreItem item : scoreList) {
                            i++;
                            if (questionList.size() < i) {
                                break;
                            }
                            ExamQuestion question = questionList.get(i - 1);
                            if (question.getTotalScore() == null || question.getTotalScore() == 0) {
                                continue;
                            }
                            JSONObject detail = new JSONObject();
                            detail.accumulate("mainNumber", question.getMainNumber());
                            detail.accumulate("subNumber", question.getSubNumber());
                            detail.accumulate("score", item.getScore());
                            detail.accumulate("marker",
                                    StringUtils.join(getMarkerNames(student, question.getMainNumber()), ","));
                            subjective.add(detail);
                        }
                        obj.accumulate("subjectiveScoreDetail", subjective);
                    }
                    // 返回评卷标记
                    if (withMarkTrack != null && withMarkTrack.booleanValue()) {
                        obj.accumulate("tags", studentService.buildSheetTags(student));
                    }
                    array.add(obj);
                } catch (Exception e) {
                    logger.error("student api error", e);
                }
            }
        }
        return array;
    }

    private List<String> getMarkerNames(ExamStudent student, Integer groupNumber) {
        List<String> names = new LinkedList<>();
        List<MarkLibrary> list = libraryService.findByStudentAndGroup(student.getId(), groupNumber);
        for (MarkLibrary library : list) {
            Marker marker = markerService.findById(library.getMarkerId());
            if (marker != null) {
                names.add(marker.getName());
            }
        }
        return names;
    }

    /**
     * 
     * @param s
     *            需要转换的字符串
     * @param convert
     *            是否正向转换
     * @return
     */
    @SuppressWarnings("unused")
    private String convert(String s, String[] diploma, String[] bachelorDegree, boolean convert) {
        if (diploma == null || bachelorDegree == null) {
            return s;
        }

        char[] ss = s.toCharArray();
        String[] str = new String[ss.length];
        for (int i = 0; i < str.length; i++) {
            str[i] = String.valueOf(ss[i]);
        }

        if (convert) {
            if (str[Integer.parseInt(diploma[2]) - 1].equals(diploma[1])) {
                str[Integer.parseInt(diploma[2]) - 1] = diploma[0];
            }
            if (str[Integer.parseInt(diploma[3]) - 1].equals(diploma[1])) {
                str[Integer.parseInt(diploma[3]) - 1] = diploma[0];
            }
            if (str[Integer.parseInt(bachelorDegree[2]) - 1].equals(bachelorDegree[1])) {
                str[Integer.parseInt(bachelorDegree[2]) - 1] = bachelorDegree[0];
            }
            if (str[Integer.parseInt(bachelorDegree[3]) - 1].equals(bachelorDegree[1])) {
                str[Integer.parseInt(bachelorDegree[3]) - 1] = bachelorDegree[0];
            }
        } else {
            if (str[Integer.parseInt(diploma[2]) - 1].equals(diploma[0])) {
                str[Integer.parseInt(diploma[2]) - 1] = diploma[1];
            }
            if (str[Integer.parseInt(diploma[3]) - 1].equals(diploma[0])) {
                str[Integer.parseInt(diploma[3]) - 1] = diploma[1];
            }
            if (str[Integer.parseInt(bachelorDegree[2]) - 1].equals(bachelorDegree[0])) {
                str[Integer.parseInt(bachelorDegree[2]) - 1] = bachelorDegree[1];
            }
            if (str[Integer.parseInt(bachelorDegree[3]) - 1].equals(bachelorDegree[0])) {
                str[Integer.parseInt(bachelorDegree[3]) - 1] = bachelorDegree[1];
            }
        }
        StringBuffer sb = new StringBuffer();
        for (String i : str) {
            sb.append(i);
        }
        return sb.toString();
    }

    @RequestMapping(value = "/score/school/{schoolId}", method = RequestMethod.POST)
    @ResponseBody
    public String getScore(@PathVariable Integer schoolId, @RequestParam String studentCode,
            @RequestParam String subjectCode, @RequestParam(required = false, defaultValue = "true") boolean encrypt,
            @RequestParam(required = false) String examSeqCode) {
        JSONObject obj = new JSONObject();
        ExamStudent student = null;
        try {
            if (Strings.isNullOrEmpty(examSeqCode)) {
                student = studentService.findBySchoolIdAndSubjectCodeAndStudentCode(schoolId, subjectCode, studentCode);
            } else {
                student = studentService.findBySchoolIdAndSubjectCodeAndStudentCodeAndRemark(schoolId, subjectCode,
                        studentCode, examSeqCode);
            }
            if (student != null) {
                DecimalFormat df = new DecimalFormat("###.#");
                obj.accumulate("exist", true);
                obj.accumulate("objectiveScore", df.format(student.getObjectiveScore()));
                obj.accumulate("subjectiveScore", df.format(student.getSubjectiveScore()));
                obj.accumulate("totalScore", df.format(student.getTotalScore()));
            } else {
                obj.accumulate("exist", false);
                obj.accumulate("objectiveScore", 0);
                obj.accumulate("subjectiveScore", 0);
                obj.accumulate("totalScore", 0);
            }
            if (!Strings.isNullOrEmpty(examSeqCode)) {
                obj.accumulate("examSeqCode", examSeqCode);
            }
            obj.accumulate("studentCode", studentCode);
            obj.accumulate("subjectCode", subjectCode);
        } catch (Exception e) {
            e.printStackTrace();
            obj.accumulate("exception", "500");
        }
        String result = obj.toString();
        return encrypt ? AESUtil.encrypt(result) : result;
    }

}
