# ************************************************************
# Database: stmms_fenti
# Generation Time: 2017-10-31 06:43:10 +0000
# ************************************************************


# Dump of table b_campus
# ------------------------------------------------------------

DROP TABLE IF EXISTS `b_campus`;

CREATE TABLE `b_campus` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `school_id` int(11) NOT NULL COMMENT '学校ID',
  `name` varchar(64) NOT NULL COMMENT '名称',
  `province` varchar(16) DEFAULT NULL COMMENT '省份',
  `city` varchar(16) DEFAULT NULL COMMENT '城市',
  `district` varchar(16) DEFAULT NULL COMMENT '地区',
  `address` varchar(128) DEFAULT NULL COMMENT '地址',
  `phone` varchar(32) DEFAULT NULL COMMENT '电话',
  `description` varchar(256) DEFAULT NULL COMMENT '学校描述',
  PRIMARY KEY (`id`),
  KEY `index1` (`school_id`,`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='学习中心表';



# Dump of table b_school
# ------------------------------------------------------------

DROP TABLE IF EXISTS `b_school`;

CREATE TABLE `b_school` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `name` varchar(64) NOT NULL COMMENT '名称',
  `province` varchar(16) NOT NULL COMMENT '省份',
  `city` varchar(16) NOT NULL COMMENT '城市',
  `address` varchar(128) DEFAULT NULL COMMENT '地址',
  `phone` varchar(32) DEFAULT NULL COMMENT '电话',
  `logo_url` varchar(64) DEFAULT NULL COMMENT '图片地址',
  `description` varchar(256) DEFAULT NULL COMMENT '描述',
  `enable` tinyint(1) NOT NULL COMMENT '是否禁用',
  `time_create` datetime NOT NULL COMMENT '创建时间',
  `time_modified` datetime DEFAULT NULL COMMENT '修改时间',
  PRIMARY KEY (`id`),
  KEY `area` (`province`,`city`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='学校表';



# Dump of table b_tag
# ------------------------------------------------------------

DROP TABLE IF EXISTS `b_tag`;

CREATE TABLE `b_tag` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `name` varchar(128) NOT NULL COMMENT '标记类型',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='标记类型表';

LOCK TABLES `b_tag` WRITE;

INSERT INTO `b_tag` (`id`, `name`)
VALUES
	(1,'试卷雷同'),
	(2,'科目错误'),
	(3,'贴错条码');

UNLOCK TABLES;


# Dump of table b_user
# ------------------------------------------------------------

DROP TABLE IF EXISTS `b_user`;

CREATE TABLE `b_user` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `login_name` varchar(64) NOT NULL COMMENT '登录名',
  `name` varchar(64) NOT NULL COMMENT '名称',
  `password` varchar(128) NOT NULL COMMENT '密码',
  `role` varchar(16) NOT NULL COMMENT '角色',
  `enable` tinyint(1) NOT NULL COMMENT '是否启用',
  `school_id` int(11) NOT NULL COMMENT '所属学校ID',
  `subject_code` varchar(32) DEFAULT NULL COMMENT '绑定科目代码',
  `last_login_time` datetime DEFAULT NULL COMMENT '最后一次登录时间',
  `last_login_ip` varchar(64) DEFAULT NULL COMMENT '最后一次登录IP',
  `created_time` datetime NOT NULL COMMENT '创建时间',
  `updated_time` datetime DEFAULT NULL COMMENT '修改时间',
  PRIMARY KEY (`id`),
  KEY `index1` (`login_name`),
  KEY `index2` (`school_id`,`enable`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='用户表';

LOCK TABLES `b_user` WRITE;

INSERT INTO `b_user` (`id`, `login_name`, `name`, `password`, `role`, `enable`, `school_id`, `subject_code`, `last_login_time`, `last_login_ip`, `created_time`, `updated_time`)
VALUES
	(1,'admin','超级管理员','f4f49b97cf567ca04b9bffdfc11d59ee','SYS_ADMIN',1,0,NULL,'2017-09-25 10:07:37','192.168.1.94','2013-10-16 18:11:51','2017-09-25 10:07:37');

UNLOCK TABLES;


# Dump of table eb_exam
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_exam`;

CREATE TABLE `eb_exam` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `name` varchar(64) NOT NULL COMMENT '考试名称',
  `school_id` int(11) NOT NULL COMMENT '学校ID',
  `exam_time` datetime NOT NULL COMMENT '考试时间',
  `status` varchar(16) NOT NULL COMMENT '状态',
  `description` varchar(128) DEFAULT NULL COMMENT '描述',
  `force_special_tag` tinyint(1) NOT NULL COMMENT '强制标记',
  `slice_config` text DEFAULT NULL COMMENT '裁切图配置',
  `creator_id` int(11) NOT NULL COMMENT '创建人',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `update_time` datetime DEFAULT NULL COMMENT '修改时间',
  `excellent_score` double DEFAULT NULL COMMENT '优秀分',
  `pass_score` double DEFAULT NULL COMMENT '及格分',
  `need_calculate` tinyint(1) NOT NULL COMMENT '需要统分',
  `process` double DEFAULT NULL COMMENT '统分进度',
  PRIMARY KEY (`id`),
  KEY `index1` (`school_id`,`create_time`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='考试表';



# Dump of table eb_exam_package
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_exam_package`;

CREATE TABLE `eb_exam_package` (
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `code` varchar(128) NOT NULL COMMENT '试卷袋编号',
  `pic_count` int(11) NOT NULL COMMENT '上传图片数量',
  PRIMARY KEY (`exam_id`,`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='卷袋信息表';



# Dump of table eb_exam_question
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_exam_question`;

CREATE TABLE `eb_exam_question` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `subject_code` varchar(32) NOT NULL DEFAULT '' COMMENT '科目代码',
  `group_number` int(11) NOT NULL COMMENT '分组序号',
  `main_number` int(11) NOT NULL COMMENT '大题号',
  `sub_number` int(11) NOT NULL COMMENT '小题号',
  `is_objective` tinyint(1) NOT NULL COMMENT '是否客观题',
  `main_title` varchar(32) NOT NULL COMMENT '大题名称',
  `answer` varchar(16) DEFAULT NULL COMMENT '正确答案',
  `total_score` double NOT NULL COMMENT '满分',
  `interval_score` double NOT NULL COMMENT '评卷间隔分',
  `total_count` int(11) DEFAULT NULL COMMENT '总人数',
  `paper_type` varchar(32) DEFAULT NULL COMMENT '试卷类型',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`,`is_objective`,`main_number`,`sub_number`,`paper_type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='小题信息表';



# Dump of table eb_exam_student
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_exam_student`;

CREATE TABLE `eb_exam_student` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `school_id` int(11) NOT NULL COMMENT '学校ID',
  `campus_name` varchar(64) NOT NULL COMMENT '学习中心名称',
  `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
  `subject_name` varchar(32) NOT NULL COMMENT '科目名称',
  `exam_number` varchar(64) NOT NULL COMMENT '准考证号',
  `student_code` varchar(64) NOT NULL COMMENT '学号',
  `name` varchar(32) NOT NULL COMMENT '姓名',
  `package_code` varchar(128) DEFAULT NULL COMMENT '试卷袋编号',
  `exam_site` varchar(64) DEFAULT NULL COMMENT '考点',
  `exam_room` varchar(64) DEFAULT NULL COMMENT '考场',
  `remark` varchar(255) DEFAULT NULL COMMENT '备注',
  `batch_code` varchar(32) DEFAULT NULL COMMENT '扫描批次号',
  `sheet_count` int(11) NOT NULL COMMENT '原图数量',
  `slice_count` int(11) NOT NULL COMMENT '小图数量',
  `answers` text COMMENT '客观题识别结果',
  `is_upload` tinyint(1) NOT NULL COMMENT '是否已上传',
  `is_absent` tinyint(1) NOT NULL COMMENT '是否缺考',
  `is_manual_absent` tinyint(1) NOT NULL COMMENT '是否人工指定缺考',
  `is_breach` tinyint(1) NOT NULL COMMENT '是否违纪',
  `is_exception` tinyint(1) NOT NULL COMMENT '是否数据异常',
  `upload_time` datetime DEFAULT NULL COMMENT '上传时间',
  `objective_score` double DEFAULT NULL COMMENT '客观总分',
  `subjective_score` double DEFAULT NULL COMMENT '主观总分',
  `objective_score_list` text COMMENT '客观得分明细',
  `subjective_score_list` text COMMENT '主观得分明细',
  `subject_level` varchar(64) DEFAULT NULL COMMENT '层次',
  `subject_category` varchar(64) DEFAULT NULL COMMENT '专业类型',
  `paper_type` varchar(32) DEFAULT NULL COMMENT '试卷类型',
  `college` varchar(64) DEFAULT NULL COMMENT '学院',
  `class_name` varchar(64) DEFAULT NULL COMMENT '班级',
  `teacher` varchar(64) DEFAULT NULL COMMENT '任课老师',
  PRIMARY KEY (`id`),
  UNIQUE KEY `index1` (`exam_id`,`exam_number`),
  KEY `index2` (`exam_id`,`student_code`),
  KEY `index3` (`exam_id`,`subject_code`,`upload_time`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='考试考生库';



# Dump of table eb_exam_subject
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_exam_subject`;

CREATE TABLE `eb_exam_subject` (
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `code` varchar(32) NOT NULL COMMENT '科目代码',
  `name` varchar(32) NOT NULL COMMENT '科目名称',
  `level` varchar(64) DEFAULT NULL COMMENT '层次',
  `category` varchar(64) DEFAULT NULL COMMENT '专业类型',
  `objective_score` double NOT NULL COMMENT '客观题满分',
  `subjective_score` double NOT NULL COMMENT '主观题满分',
  `total_score` double NOT NULL COMMENT '全卷满分',
  `upload_count` int(11) NOT NULL COMMENT '已上传人数',
  `has_answer` tinyint(1) NOT NULL COMMENT '是否上传答案',
  `has_paper` tinyint(1) NOT NULL COMMENT '是否上传试卷',
  `remark` varchar(255) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`exam_id`,`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='考试科目表';



# Dump of table eb_export_imglist
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_export_imglist`;

CREATE TABLE `eb_export_imglist` (
  `exam_id` int(11) NOT NULL,
  `exam_number` varchar(50) COLLATE utf8_bin NOT NULL,
  `Remark` varchar(255) COLLATE utf8_bin DEFAULT NULL,
  PRIMARY KEY (`exam_id`,`exam_number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;



# Dump of table eb_mark_group
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_mark_group`;

CREATE TABLE `eb_mark_group` (
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
  `number` int(11) NOT NULL COMMENT '序号',
  `status` varchar(16) NOT NULL COMMENT '评卷状态',
  `pic_list` varchar(255) NOT NULL COMMENT '小图配置',
  `total_score` double NOT NULL COMMENT '满分',
  `trial_count` int(11) DEFAULT NULL COMMENT '试评数量',
  `double_rate` double DEFAULT NULL COMMENT '双评比例',
  `arbitrate_threshold` double DEFAULT NULL COMMENT '仲裁阈值',
  `score_policy` varchar(16) DEFAULT NULL COMMENT '合分策略',
  `mark_mode` varchar(16) DEFAULT NULL COMMENT '强制评卷模式',
  `sheet_view` tinyint(1) NOT NULL COMMENT '是否允许查看原卷',
  `build_time` datetime DEFAULT NULL COMMENT '最后生成任务时间',
  `library_count` int(11) NOT NULL COMMENT '任务总量',
  `marked_count` int(11) NOT NULL COMMENT '已评数量',
  `left_count` int(11) NOT NULL COMMENT '剩余数量',
  PRIMARY KEY (`exam_id`,`subject_code`,`number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='评卷分组表';



# Dump of table eb_marker
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_marker`;

CREATE TABLE `eb_marker` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `subject_code` varchar(12) NOT NULL COMMENT '科目代码',
  `group_number` int(11) NOT NULL COMMENT '大题号',
  `login_name` varchar(128) NOT NULL COMMENT '登录名',
  `name` varchar(64) NOT NULL COMMENT '姓名',
  `password` varchar(128) NOT NULL COMMENT '密码',
  `enable` tinyint(1) NOT NULL COMMENT '是否禁用',
  `mode` varchar(16) DEFAULT NULL COMMENT '评卷模式',
  `top_count` int(11) DEFAULT NULL COMMENT '评卷数上限',
  `last_login_ip` varchar(64) DEFAULT NULL COMMENT '上次登录IP',
  `last_login_time` datetime DEFAULT NULL COMMENT '上次登录时间',
  PRIMARY KEY (`id`),
  KEY `index1` (`login_name`),
  KEY `index2` (`exam_id`,`subject_code`,`group_number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='评卷员表';


# Dump of table eb_marker_class
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_marker_class`;

CREATE TABLE `eb_marker_class` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `class_name` varchar(255) COLLATE utf8_bin DEFAULT NULL COMMENT '班级名称',
  `marker_id` int(11) NOT NULL COMMENT '评卷员ID',
  PRIMARY KEY (`id`),
  KEY `index1` (`marker_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='评卷员班级表';


# Dump of table m_arbitrate_history
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_arbitrate_history`;

CREATE TABLE `m_arbitrate_history` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '自增主键',
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
  `group_number` int(11) NOT NULL COMMENT '大题号',
  `exam_number` varchar(64) NOT NULL COMMENT '准考证号',
  `student_id` int(11) NOT NULL COMMENT '考生ID',
  `status` int(11) NOT NULL COMMENT '状态',
  `user_id` int(11) DEFAULT NULL COMMENT '处理人ID',
  `total_score` double DEFAULT NULL COMMENT '总分',
  `score_list` varchar(255) DEFAULT NULL COMMENT '给分明细',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `update_time` datetime DEFAULT NULL COMMENT '处理时间',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`,`group_number`,`status`),
  KEY `index2` (`user_id`,`status`,`update_time`),
  KEY `index3` (`student_id`,`status`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='仲裁记录表';


# Dump of table m_library
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_library`;

CREATE TABLE `m_library` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
  `group_number` int(11) NOT NULL COMMENT '大题号',
  `campus_id` int(11) NOT NULL COMMENT '学习中心ID',
  `student_id` int(11) NOT NULL COMMENT '考生ID',
  `exam_number` varchar(64) NOT NULL COMMENT '准考证号',
  `task_number` int(11) DEFAULT NULL COMMENT '多评任务序号',
  `marker_id` int(11) DEFAULT NULL COMMENT '评卷员ID',
  `marker_time` datetime DEFAULT NULL COMMENT '评卷时间',
  `marker_score` double DEFAULT NULL COMMENT '评卷总分',
  `marker_score_list` varchar(256) DEFAULT NULL COMMENT '评卷给分明细',
  `tags` varchar(16) DEFAULT NULL COMMENT '评卷标记',
  `status` int(11) NOT NULL COMMENT '状态',
  `header_id` int(11) DEFAULT NULL COMMENT '科组长ID',
  `header_score` double DEFAULT NULL COMMENT '科组长总分',
  `header_score_list` varchar(255) DEFAULT NULL COMMENT '科组长给分明细',
  `header_time` datetime DEFAULT NULL COMMENT '科组长评卷时间',
  `marker_spent` int(11) DEFAULT NULL COMMENT '最新评卷时间',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`,`group_number`,`status`),
  UNIQUE KEY `index2` (`student_id`,`group_number`,`task_number`),
  KEY `index3` (`marker_id`,`status`,`marker_time`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='评卷任务表';


# Dump of table m_special_tag
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_special_tag`;

CREATE TABLE `m_special_tag` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `library_id` int(11) NOT NULL COMMENT '评卷任务ID',
  `position_x` double NOT NULL COMMENT 'X轴位置',
  `position_y` double NOT NULL COMMENT 'Y轴位置',
  `tag_name` varchar(64) NOT NULL COMMENT '标记内容',
  PRIMARY KEY (`id`),
  KEY `index1` (`library_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='特殊标记表';


# Dump of table m_track
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_track`;

CREATE TABLE `m_track` (
  `library_id` int(11) NOT NULL COMMENT '评卷任务ID',
  `question_number` varchar(128) NOT NULL COMMENT '完整题号',
  `number` int(11) NOT NULL COMMENT '序号',
  `student_id` int(11) NOT NULL COMMENT '考生ID',
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
  `group_number` int(11) NOT NULL COMMENT '大题题号',
  `marker_id` int(11) NOT NULL COMMENT '评卷员ID',
  `score` double NOT NULL COMMENT '给分',
  `position_x` double NOT NULL COMMENT 'X轴位置',
  `position_y` double NOT NULL COMMENT 'Y轴位置',
  PRIMARY KEY (`library_id`,`question_number`,`number`),
  KEY `index1` (`student_id`,`group_number`),
  KEY `index2` (`marker_id`),
  KEY `index3` (`exam_id`,`subject_code`,`group_number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='轨迹给分表';


# Dump of table m_trial_history
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_trial_history`;
CREATE TABLE `m_trial_history` (
  `library_id` int(11) NOT NULL COMMENT '评卷任务ID',
  `marker_id` int(11) NOT NULL COMMENT '评卷员ID',
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
  `group_number` int(11) NOT NULL COMMENT '大题号',
  `student_id` int(11) NOT NULL COMMENT '考生ID',
  `marker_score` double NOT NULL COMMENT '评卷总分',
  `marker_score_list` varchar(255) NOT NULL COMMENT '评卷总分明细',
  `marker_time` datetime NOT NULL COMMENT '评卷时间',
  PRIMARY KEY (`library_id`,`marker_id`),
  KEY `index1` (`exam_id`,`subject_code`,`group_number`),
  KEY `index2` (`student_id`),
  KEY `index3` (`marker_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='试评记录表';


# Dump of table m_trial_library
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_trial_library`;
CREATE TABLE `m_trial_library` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
  `group_number` int(11) NOT NULL COMMENT '大题号',
  `campus_id` int(11) NOT NULL COMMENT '学习中心ID',
  `student_id` int(11) NOT NULL COMMENT '考生ID',
  `exam_number` varchar(64) NOT NULL COMMENT '准考证号',
  PRIMARY KEY (`id`),
  UNIQUE KEY `index2` (`student_id`,`group_number`) USING BTREE,
  KEY `index1` (`exam_id`,`subject_code`,`group_number`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8 COMMENT='试评任务表';


# Dump of table m_trial_tag
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_trial_tag`;
CREATE TABLE `m_trial_tag` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `library_id` int(11) NOT NULL COMMENT '评卷任务ID',
  `marker_id` int(11) NOT NULL COMMENT '评卷员ID',
  `content` varchar(16) NOT NULL COMMENT '标记内容',
  `position_x` double NOT NULL COMMENT 'X轴位置',
  `position_y` double NOT NULL COMMENT 'Y轴位置',
  PRIMARY KEY (`id`),
  KEY `index1` (`library_id`,`marker_id`),
  KEY `index2` (`marker_id`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8 COMMENT='试评特殊标记表';


# Dump of table m_trial_track
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_trial_track`;
CREATE TABLE `m_trial_track` (
  `library_id` int(11) NOT NULL COMMENT '评卷任务ID',
  `marker_id` int(11) NOT NULL COMMENT '评卷员ID',
  `question_number` varchar(32) NOT NULL COMMENT '完整题号',
  `number` int(11) NOT NULL COMMENT '序号',
  `exam_id` int(11) NOT NULL COMMENT '考试ID',
  `subject_code` varchar(255) NOT NULL COMMENT '科目代码',
  `group_number` int(11) NOT NULL COMMENT '大题题号',
  `student_id` int(11) NOT NULL COMMENT '考生ID',
  `score` double NOT NULL COMMENT '给分',
  `position_x` double NOT NULL COMMENT 'X轴位置',
  `position_y` double NOT NULL COMMENT 'Y轴位置',
  PRIMARY KEY (`library_id`,`marker_id`,`number`,`question_number`),
  KEY `index1` (`exam_id`,`subject_code`,`group_number`),
  KEY `index2` (`student_id`),
  KEY `index3` (`marker_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='试评轨迹表';


# Dump of table s_basic_group
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_group`;
CREATE TABLE `s_basic_group` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) DEFAULT NULL COMMENT '考试ID',
  `subject_code` varchar(255) DEFAULT NULL COMMENT '科目代码',
  `subject_name` varchar(255) DEFAULT NULL COMMENT '科目名称',
  `objective` tinyint(1) DEFAULT NULL COMMENT '是否客观题',
  `group_name` varchar(255) DEFAULT NULL COMMENT '大题名称',
  `group_number` int(11) DEFAULT NULL COMMENT '大题号',
  `max_score` double DEFAULT NULL COMMENT '最高分',
  `min_score` double DEFAULT NULL COMMENT '最低分',
  `avg_score` double DEFAULT NULL COMMENT '平均分',
  `total_score` double DEFAULT NULL COMMENT '满分',
  `stdev` double DEFAULT NULL COMMENT '标准差',
  `coefficient` double DEFAULT NULL COMMENT '差异系数',
  `score_rate` double DEFAULT NULL COMMENT '得分率',
  `full_count` int(11) DEFAULT NULL COMMENT '满分人数',
  `zero_count` int(11) DEFAULT NULL COMMENT '零分人数',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='大题统计表';


# Dump of table s_basic_question
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_question`;
CREATE TABLE `s_basic_question` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) DEFAULT NULL COMMENT '考试ID',
  `subject_code` varchar(255) DEFAULT NULL COMMENT '科目代码',
  `subject_name` varchar(255) DEFAULT NULL COMMENT '科目名称',
  `objective` tinyint(1) DEFAULT NULL COMMENT '是否客观题',
  `question_name` varchar(128) DEFAULT NULL COMMENT '题目名称',
  `paper_type` varchar(32) DEFAULT NULL COMMENT '试卷类型',
  `main_number` int(11) NOT NULL COMMENT '大题号',
  `sub_number` int(11) NOT NULL COMMENT '小题号',
  `avg_score` double DEFAULT NULL COMMENT '平均分',
  `total_score` double DEFAULT NULL COMMENT '满分',
  `stdev` double DEFAULT NULL COMMENT '标准差',
  `score_rate` double DEFAULT NULL COMMENT '得分率',
  `full_score_rate` double DEFAULT NULL COMMENT '满分率',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='小题统计表';


# Dump of table s_basic_subject
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_subject`;
CREATE TABLE `s_basic_subject` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) DEFAULT NULL COMMENT '考试ID',
  `subject_code` varchar(255) DEFAULT NULL COMMENT '科目代码',
  `subject_name` varchar(255) DEFAULT NULL COMMENT '科目名称',
  `absent_count` int(11) DEFAULT NULL COMMENT '缺考人数',
  `breach_count` int(11) DEFAULT NULL COMMENT '违纪人数',
  `pass_count` int(11) DEFAULT NULL COMMENT '及格人数',
  `pass_rate` double DEFAULT NULL COMMENT '及格率',
  `reality_count` int(11) DEFAULT NULL COMMENT '实考人数',
  `max_score` double DEFAULT NULL COMMENT '最高分',
  `min_score` double DEFAULT NULL COMMENT '最低分',
  `avg_score` double DEFAULT NULL COMMENT '平均分',
  `total_count` int(11) DEFAULT NULL COMMENT '总人数',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='科目统计表';


# Dump of table s_basic_subject_class
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_subject_class`;
CREATE TABLE `s_basic_subject_class` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) DEFAULT NULL COMMENT '考试ID',
  `subject_code` varchar(255) DEFAULT NULL COMMENT '科目代码',
  `subject_name` varchar(255) DEFAULT NULL COMMENT '科目名称',
  `class_name` varchar(255) DEFAULT NULL COMMENT '班级名称',
  `excellent_count` int(11) DEFAULT NULL COMMENT '优秀人数',
  `excellent_rate` double DEFAULT NULL  COMMENT '优秀率',
  `pass_count` int(11) DEFAULT NULL COMMENT '及格人数',
  `pass_rate` double DEFAULT NULL COMMENT '及格率',
  `max_score` double DEFAULT NULL COMMENT '最高分',
  `min_score` double DEFAULT NULL COMMENT '最低分',
  `avg_score` double DEFAULT NULL COMMENT '平均分',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='班级统计表';


# Dump of table s_basic_subject_college
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_subject_college`;
CREATE TABLE `s_basic_subject_college` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) DEFAULT NULL COMMENT '考试ID',
  `subject_code` varchar(255) DEFAULT NULL COMMENT '科目代码',
  `subject_name` varchar(255) DEFAULT NULL COMMENT '科目名称',
  `college_name` varchar(255) DEFAULT NULL COMMENT '学院名称',
  `excellent_count` int(11) DEFAULT NULL COMMENT '优秀人数',
  `excellent_rate` double DEFAULT NULL COMMENT '优秀率',
  `pass_count` int(11) DEFAULT NULL COMMENT '及格人数',
  `pass_rate` double DEFAULT NULL COMMENT '及格率',
  `max_score` double DEFAULT NULL COMMENT '最高分',
  `min_score` double DEFAULT NULL COMMENT '最低分',
  `avg_score` double DEFAULT NULL COMMENT '平均分',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='学院统计表';


# Dump of table s_basic_subject_teacher
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_subject_teacher`;
CREATE TABLE `s_basic_subject_teacher` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) DEFAULT NULL COMMENT '考试ID',
  `subject_code` varchar(255) DEFAULT NULL COMMENT '科目代码',
  `subject_name` varchar(255) DEFAULT NULL COMMENT '科目名称',
  `teacher_name` varchar(255) DEFAULT NULL COMMENT '任课老师',
  `excellent_count` int(11) DEFAULT NULL COMMENT '优秀人数',
  `excellent_rate` double DEFAULT NULL COMMENT '优秀率',
  `pass_count` int(11) DEFAULT NULL COMMENT '及格人数',
  `pass_rate` double DEFAULT NULL COMMENT '及格率',
  `max_score` double DEFAULT NULL COMMENT '最高分',
  `min_score` double DEFAULT NULL COMMENT '最低分',
  `avg_score` double DEFAULT NULL COMMENT '平均分',
  `reality_count` int(11) DEFAULT NULL COMMENT '实考人数',
  `relative_avg_score` double DEFAULT NULL COMMENT '平均相对分',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='任课老师统计表';


# Dump of table s_basic_subject_teacher_class
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_subject_teacher_class`;
CREATE TABLE `s_basic_subject_teacher_class` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) DEFAULT NULL COMMENT '考试ID',
  `subject_code` varchar(255) DEFAULT NULL COMMENT '科目代码',
  `subject_name` varchar(255) DEFAULT NULL COMMENT '科目名称',
  `teacher_name` varchar(255) DEFAULT NULL COMMENT '任课老师',
  `class_name` varchar(255) DEFAULT NULL COMMENT '班级名称',
  `excellent_count` int(11) DEFAULT NULL COMMENT '优秀人数',
  `excellent_rate` double DEFAULT NULL COMMENT '优秀率',
  `pass_count` int(11) DEFAULT NULL COMMENT '及格人数',
  `pass_rate` double DEFAULT NULL COMMENT '及格率',
  `max_score` double DEFAULT NULL COMMENT '最高分',
  `min_score` double DEFAULT NULL COMMENT '最低分',
  `avg_score` double DEFAULT NULL COMMENT '平均分',
  `reality_count` int(11) DEFAULT NULL COMMENT '实考人数',
  `relative_avg_score` double DEFAULT NULL COMMENT '平均相对分',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='任课老师班级统计表';


# Dump of table s_range_subject
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_range_subject`;
CREATE TABLE `s_range_subject` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `exam_id` int(11) DEFAULT NULL COMMENT '考试ID',
  `subject_code` varchar(255) DEFAULT NULL COMMENT '科目代码',
  `subject_name` varchar(255) DEFAULT NULL COMMENT '科目名称',
  `subject_level` varchar(255) DEFAULT NULL COMMENT '科目层次',
  `totalCount` int(11) DEFAULT NULL COMMENT '总人数',
  `count0_49` int(11) DEFAULT NULL COMMENT '0-49',
  `count50_59` int(11) DEFAULT NULL COMMENT '50-59',
  `count60_69` int(11) DEFAULT NULL COMMENT '60-69',
  `count70_79` int(11) DEFAULT NULL COMMENT '70-79',
  `count80_89` int(11) DEFAULT NULL COMMENT '80-89',
  `count90_100` int(11) DEFAULT NULL COMMENT '90-100',
  `lt60` int(11) DEFAULT NULL COMMENT '<60',
  `mte60` int(11) DEFAULT NULL COMMENT '>=60',
  `percent0_49` double NOT NULL COMMENT '0-49占比',
  `percent50_59` double NOT NULL COMMENT '50-59占比',
  `percent60_69` double NOT NULL COMMENT '60-69占比',
  `percent70_79` double NOT NULL COMMENT '70-79占比',
  `percent80_89` double NOT NULL COMMENT '80-89占比',
  `percent90_100` double NOT NULL COMMENT '90-100占比',
  `percent_lt60` double NOT NULL COMMENT '<60占比',
  `percent_mte60` double NOT NULL COMMENT '>=60占比',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`,`subject_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='分段统计表';